#include <iostream>
#include <string>
#include <cstdlib>
#include <fstream>
#include <cstdio>
#include <pthread.h>
#include "clustering.h"
#include "mypdbreader/distance.h"

/**
 * \brief Clustering::Clustering()
 * Constructeur d'une operation de clustering
 * \param info Class représentant toute les info sur les proteine a traiter
 * \param debug Token permetant d'activer le debug un fichier de log sera crée pour stocker le debug
 */

Clustering::Clustering( reader * info, bool debug)
{
    if(info == NULL)
    {
        exit(-1);
    }
    this->info = info;
    std::ofstream myfile;

    //////// INITIALISATION///////////
    mes_cluster_tab = new Ensemble_Cluster[info->getNbProteine()];
    for(unsigned int i = 0; i < info->getNbProteine(); i++)
    {
        mes_cluster_tab[i].setparent(this);
    }

    matriceRMSD = new double*[info->getNbProteine()];
    for (unsigned int i = 0; i < info->getNbProteine(); i++)
        matriceRMSD[i] = new double[info->getNbProteine()];


    ////////////////////CALCUL DU RMSD ENTRE TOUTE LES PROTEINE /////////////////
    if(debug)
    {
        myfile.open ("log");
        myfile << "Nombre de Proteine recenser :" << info->getNbProteine()<< std::endl;
        myfile << "Matrice RMSD :" << std::endl << std::endl;
    }

    for(unsigned int i = 0; i < info->getNbProteine(); i++)
    {
        // marquage de l'enplacement de la proteine sur la matrice du RMSD
        info->getProteineAt(i).setID(i);

        if(debug)
        {
            myfile << "[";
        }
        for(unsigned int u = 0; u < info->getNbProteine(); u++)
        {
            matriceRMSD[i][u] = calculRMSD(info->getProteineAt(i),info->getProteineAt(u));

            if(debug)
            {
                myfile << matriceRMSD[i][u] << ",";
            }
        }
        if(debug)
        {
            myfile << "]"<< std::endl;
        }

    }


    ///////////////CHARGEMENT DES PROTEINE DANS LES CLUSTER///////////////////////
    for(unsigned int i = 0; i < info->getNbProteine(); i++ )
    {
        Cluster *cluster = new Cluster(i,this);
        mes_cluster_tab[0].addCluster(cluster);
        cluster->addProtein(&(info->getProteineAt(i)));
    }


    /////////////ON FUSIONNE LES CLUSTER POUR FAIRE CHAQUE ETAPE///////////
    for(unsigned int i = 1; i < info->getNbProteine(); i++ )
    {
        mes_cluster_tab[i] = mes_cluster_tab[i-1];
        if(debug)
        {
            myfile << "-------FUSION DES CLUSTER ITERATION "<< i <<"---------" << std::endl;
            mes_cluster_tab[i].fusionnerClusterPlusProche(&myfile);
            mes_cluster_tab[i].calc_AverageSpread();
            myfile << "Average spread : " << mes_cluster_tab[i].getAverageSpread() << std::endl;
        }
        else
        {
            mes_cluster_tab[i].fusionnerClusterPlusProche();
            mes_cluster_tab[i].calc_AverageSpread();
        }
    }
    bool premier_tour = true;
    double minaverage = 0;
    double maxaverage = 0;

    ////////////ON CALCUL LE MIN/MAX de L'average SPREAD de tout les ensemble de cluster
    for(unsigned int i = 1; i < info->getNbProteine(); i++ )
    {

        if(mes_cluster_tab[i].getAverageSpread() < minaverage || premier_tour == true)
        {
            minaverage = mes_cluster_tab[i].getAverageSpread();
            premier_tour = false;
        }
        if(mes_cluster_tab[i].getAverageSpread() > maxaverage)
        {
            maxaverage = mes_cluster_tab[i].getAverageSpread();
        }
    }
    if(debug)
    {
        myfile << "MIN AVERAGE : " << minaverage << std::endl;
        myfile << "MAX AVERAGE : " << maxaverage << std::endl;
    }
    if((minaverage == maxaverage) && minaverage == 0)
    {
        std::cerr << "Aucune dispertion donc les proteine sont les meme"<< std::endl;
        return;
    }

    double AverageSpreadNormaliser;
    int cutoffpoint;
    double penaltyValue = 0;
    premier_tour = true;
    for(unsigned int i = 1; i < info->getNbProteine()-1; i++ )
    {
        AverageSpreadNormaliser = ((info->getNbProteine()-2)*mes_cluster_tab[i].getAverageSpread()) - ((info->getNbProteine()-2)*minaverage) +  (maxaverage-minaverage);
        AverageSpreadNormaliser = AverageSpreadNormaliser / (maxaverage-minaverage);

        double Pi = AverageSpreadNormaliser + mes_cluster_tab[i].getNbCluster();

        if(debug)
        {
            myfile << "Average normaliser iteration " << i <<" : " << AverageSpreadNormaliser << std::endl;
            myfile << "Penalty value iteration "<< i <<" : " << Pi << std::endl;
        }


        if(Pi < penaltyValue || premier_tour)
        {
            penaltyValue = Pi;
            cutoffpoint = i;
            premier_tour = false;
        }
    }

    this->cutoffPoint = cutoffpoint;

    if(debug)
    {
        myfile << "Cut of point selectionner : " << cutoffpoint << std::endl;
        myfile.close();
    }

}
/**
 * \brief Clustering::~Clustering()
 * Destructeur d'une operation de clustering
 */
Clustering::~Clustering()
{

    for(unsigned int i = 0; i < info->getNbEntryPDB(); ++i) {
        delete [] matriceRMSD[i];
    }
    delete [] matriceRMSD;
    delete [] mes_cluster_tab;
}

/**
 * \brief Clustering::getReaderfromPdbFile()
 * Récupere toute les info d'un fichier PDB et recupere une instance du lecteur d'information
 * \param chemin Chemin du fichier PDB
 * \return Une instance avec toute les information du fichier PDB
 */
reader * Clustering::getReaderfromPdbFile(std::string chemin)
{
    std::ifstream in(chemin.c_str());
    if (!in)
    {
        std::cerr << "Error opening input file: " << chemin << std::endl;
        return NULL;
    }

    reader * fichierPDB = new reader();
    fichierPDB->readPDB(in);
    return fichierPDB;
}
/**
 * \brief Clustering::getCutoffPoint()
 * Recupere le cutoffPoint du resultat du clustering
 * \return Retourne la valeur du cutoff Point
 */

int Clustering::getCutoffPoint()
{
    return this->cutoffPoint;
}


/**
 * \brief Clustering::printResultText()
 * Permet d'afficher le resultat de la clusterisation de maniere textuelle
 */
void Clustering::printResultText()
{
    mes_cluster_tab[this->cutoffPoint].afficherText();
}

/**
 * \brief Clustering::printResultText()
 * Permet d'ecrire le resultat de la clusterisation de maniere textuelle dans un fichier
 * \param fichier Fichier ou enregistrer le resultat
 */
void Clustering::printResultText(std::string fichier)
{
    std::ofstream myfile;
    myfile.open (fichier.c_str());
    mes_cluster_tab[this->cutoffPoint].ecrireText(myfile);
    myfile.close();
}
/**
 * \brief Clustering::printResultTab()
 * Permet d'afficher le resultat de la clusterisation en tableau de style [[2,3],[4]]
 */
void Clustering::printResultTab()
{
    mes_cluster_tab[this->cutoffPoint].afficherTab();
}
/**
 * \brief Clustering::printResultTab()
 * Permet d'ecrire le resultat de la clusterisation en tableau de style [[2,3],[4]] dans un fichier
 * \param fichier Fichier ou enregistrer le resultat
 */
void Clustering::printResultTab(std::string fichier)
{
    std::ofstream myfile;
    myfile.open (fichier.c_str());
    mes_cluster_tab[this->cutoffPoint].ecrireTab(myfile);
    myfile.close();
}

/**
 * \brief Clustering::getRMSD()
 * Permet de recuperer une valeur de RMSD pour un emplacement donner
 * la ligne ou la collone ou se trouve une proteine est aussi sont identifiant :)
 * \param x Ligne de la premiere proteine
 * \param y Collonde de la seconde proteine
 * \return La distance RMSD entre les deux proteine
 */
double Clustering::getRMSD(int x,int y)
{
    return matriceRMSD[x][y];
}

