#include "clustering.h"
#include <iostream>
#include <fstream>
/**
 * \brief Ensemble_Cluster::Ensemble_Cluster()
 * Constructeur d'un ensemble de cluster
 * \param parent Pointeur vers le pere de l'ensemble
 */
Ensemble_Cluster::Ensemble_Cluster(Clustering *parent)
{
    this->parent = parent;
    averageSpread = 0;
}

/**
 * \brief Ensemble_Cluster::addCluster()
 * Ajoute un cluster a l'ensemble de cluster
 * \param var Pointeur vers le cluster a ajouter
 */
void Ensemble_Cluster::addCluster(Cluster * var)
{
    //on ajoute l'adresse de la proteine dans le cluster
    m_cluster.push_back(var);
}


/**
 * \brief Ensemble_Cluster::setparent()
 * Permet de redefinir le parent de l'ensemble de cluster
 * \param parent Pointeur vers le parent
 */
void Ensemble_Cluster::setparent(Clustering *parent)
{
    this->parent = parent;
}

/**
 * \brief Ensemble_Cluster::getNbCluster()
 * Permet d'obtenir le nombre de cluster dans l'ensemble
 * \return Le nombre de cluster
 */
int Ensemble_Cluster::getNbCluster()
{
    return this->m_cluster.size();
}


/**
 * \brief Ensemble_Cluster::getAverageSpread()
 * Permet d'obtenir l'average spread de l'ensemble de cluster
 * \return L'average spread
 */
double Ensemble_Cluster::getAverageSpread()
{
    return this->averageSpread;
}
/**
 * \brief Ensemble_Cluster::calc_AverageSpread()
 * Permet de recalculer l'average spread des membre du cluster
 */
void Ensemble_Cluster::calc_AverageSpread()
{
    int cnum = 0;
    averageSpread = 0;
    for(unsigned int i = 0 ; i < this->m_cluster.size();i++)
    {
        if(this->m_cluster.at(i)->getnbProtein() > 1)
        {
            cnum++;
            averageSpread += this->m_cluster.at(i)->getSpread();
        }
    }
    averageSpread = averageSpread / cnum;
    //std::cout << "average spread : "<< averageSpread <<std::endl;
}

/**
 * \brief Ensemble_Cluster::operator=()
 * Surcharge de l'operateur par copie, permet de copier 2 ensemble de cluster
 * \param var Reference vers un autre ensemble de cluster
 * \return un ensemble de cluster
 */
Ensemble_Cluster& Ensemble_Cluster::operator=(const Ensemble_Cluster& var)
{
    this->parent = var.parent;
    for(unsigned int i = 0; i < var.m_cluster.size(); i++)
    {
        this->m_cluster.push_back(new Cluster(*(var.m_cluster.at(i))));
    }
    return *this;
}
/**
 * \brief Ensemble_Cluster::getDistance()
 * Récupere la distance entre deux cluster
 * \param clu1 Pointeur vers le premier cluster
 * \param clu2 Pointeur vers le second cluster
 * \return Distance entre les deux cluster
 */
double Ensemble_Cluster::getDistance(Cluster * clu1,Cluster * clu2)
{
    double var = 0;
    for(unsigned int k = 0; k < clu1->getnbProtein(); k++)
    {
        for(unsigned int i = 0; i < clu2->getnbProtein(); i++)
        {
            var += this->parent->getRMSD(clu1->proteinAt(k)->getID(),clu2->proteinAt(i)->getID());
        }
    }
    var /= (clu1->getnbProtein()*clu2->getnbProtein());
    if(var == 0)
    {
        var = var + 0;
    }
    return var;
}
/**
 * \brief Ensemble_Cluster::fusionnerClusterPlusProche()
 * Fonction qui permet de fusionner les deux cluster les plus proche de l'ensemble de cluster
 */
void Ensemble_Cluster::fusionnerClusterPlusProche(std::ofstream *debug)
{
    int clu1_pris = 0;
    int clu2_pris = 0;
    double result = -1;
    for(unsigned int clu1 = 0; clu1 < this->m_cluster.size(); clu1++)
    {
        for(unsigned int clu2 = 0; clu2 < this->m_cluster.size(); clu2++)
        {
            if(clu1 != clu2)
            {
                double var = getDistance(this->m_cluster.at(clu1),this->m_cluster.at(clu2));
                if(result == -1)
                {
                    clu1_pris = clu1;
                    clu2_pris = clu2;
                    result = var;
                }
                if(var < result)
                {
                    clu1_pris = clu1;
                    clu2_pris = clu2;
                    result = var;
                }
            }
        }
    }
    
    Cluster *cluster1 = this->m_cluster.at(clu1_pris);
    Cluster *cluster2 = this->m_cluster.at(clu2_pris);

    if(debug != NULL)
    {
        *debug << "Fusion du cluster : " << (clu1_pris + 1) << " et " << (clu2_pris + 1) << "dist :" << getDistance(cluster1,cluster2) << std::endl;
    }
	

    for(unsigned int i = 0 ; i < cluster2->getnbProtein();i++)
	{
        //std::cout << "cluster 2 nb prot : " << cluster2->getnbProtein() << std::endl;
        cluster1->addProtein(cluster2->proteinAt(i));
	}

    // on recalcul le spread;
    cluster1->calculSpread();

    if(debug != NULL)
    {
        *debug << "Spread du nouveau cluster : " << cluster1->getSpread() << std::endl;
    }
    this->m_cluster.erase(this->m_cluster.begin()+clu2_pris);
}
/**
 * \brief Ensemble_Cluster::afficherText()
 * Fonction qui permet d'afficher en texte la situation de l'ensemble de cluster
 * Cela afficher pour chaque cluster, toute les proteine et leur ID
 */
void Ensemble_Cluster::afficherText()
{
    for(unsigned int clu1 = 0; clu1 < this->m_cluster.size(); clu1++)
    {
        std::cout << "cluster no " << clu1 << " :" << std::endl;
        for(unsigned int cluster = 0; cluster < this->m_cluster.at(clu1)->getnbProtein(); cluster++)
        {
            std::cout << "Proteine no " << this->m_cluster.at(clu1)->proteinAt(cluster)->getID()+1 << std::endl;
        }
    }
}
/**
 * \brief Ensemble_Cluster::ecrireText()
 * Fonction qui permet d'ecrire en texte la situation de l'ensemble de cluster
 * Cela ecrit dans un flux pour chaque cluster, toute les proteine et leur ID
 * \param flux Flux ou seront envoiyer les données
 */
void Ensemble_Cluster::ecrireText(std::ofstream& flux)
{
    for(unsigned int clu1 = 0; clu1 < this->m_cluster.size(); clu1++)
    {
        flux << "cluster no " << clu1 << " :" << std::endl;
        for(unsigned int cluster = 0; cluster < this->m_cluster.at(clu1)->getnbProtein(); cluster++)
        {
            flux << "Proteine no " << this->m_cluster.at(clu1)->proteinAt(cluster)->getID()+1 << std::endl;
        }
    }
}

/**
 * \brief Ensemble_Cluster::ecrireText()
 * Fonction qui permet d'ecrire en texte la situation de l'ensemble de cluster
 * Cela ecrit dans un fichier pour chaque cluster, toute les proteine et leur ID
 * \param fichier Chemin du fichier ou seront ecrit les données
 */
void Ensemble_Cluster::ecrireText(std::string fichier)
{
    std::ofstream myfile;
    myfile.open (fichier.c_str());
    for(unsigned int clu1 = 0; clu1 < this->m_cluster.size(); clu1++)
    {
        myfile << "cluster no " << clu1 << " :" << std::endl;
        for(unsigned int cluster = 0; cluster < this->m_cluster.at(clu1)->getnbProtein(); cluster++)
        {
            myfile << "Proteine no " << this->m_cluster.at(clu1)->proteinAt(cluster)->getID()+1 << std::endl;
        }
    }
    myfile.close();
}

/**
 * \brief Ensemble_Cluster::afficherTab()
 * Fonction qui permet d'afficher en texte la situation de l'ensemble de cluster
 * Cela afficher pour chaque cluster, toute les proteine et leur ID sous format [[1,2],[3]]
 */
void Ensemble_Cluster::afficherTab()
{
    std::cout << "[";
    for(unsigned int cluster = 0; cluster < this->m_cluster.size(); cluster++)
    {
        std::cout << "[";
        for(unsigned int proteine = 0; proteine < this->m_cluster.at(cluster)->getnbProtein(); proteine++)
        {
            if((proteine+1) < this->m_cluster.at(cluster)->getnbProtein())
            {
                std::cout << this->m_cluster.at(cluster)->proteinAt(proteine)->getID()+1 << ",";
            }
            else
            {
                std::cout << this->m_cluster.at(cluster)->proteinAt(proteine)->getID()+1;
            }
        }
        std::cout << "]";
        if((cluster+1) < this->m_cluster.size())
        {
            std::cout << ",";
        }
    }
    std::cout << "]";
}
/**
 * \brief Ensemble_Cluster::ecrireTab()
 * Fonction qui permet d'ecrire en texte la situation de l'ensemble de cluster
 * Cela ecrit dans un fichier pour chaque cluster, toute les proteine et leur ID sous format [[1,2],[3]]
 * \param fichier Chemin du fichier ou seront ecrit les données
 */
void Ensemble_Cluster::ecrireTab(std::string fichier)
{
    std::ofstream myfile;
    myfile.open (fichier.c_str());
    myfile << "[";
    for(unsigned int cluster = 0; cluster < this->m_cluster.size(); cluster++)
    {
        myfile << "[";
        for(unsigned int proteine = 0; proteine < this->m_cluster.at(cluster)->getnbProtein(); proteine++)
        {
            if((proteine+1) < this->m_cluster.at(cluster)->getnbProtein())
            {
                myfile << this->m_cluster.at(cluster)->proteinAt(proteine)->getID()+1 << ",";
            }
            else
            {
                myfile << this->m_cluster.at(cluster)->proteinAt(proteine)->getID()+1;
            }
        }
        myfile << "]";
        if((cluster+1) < this->m_cluster.size())
        {
            myfile << ",";
        }
    }
    myfile << "]";
    myfile.close();
}
/**
 * \brief Ensemble_Cluster::ecrireTab()
 * Fonction qui permet d'ecrire en texte la situation de l'ensemble de cluster
 * Cela ecrit dans un flux pour chaque cluster, toute les proteine et leur ID sous format [[1,2],[3]]
 * \param flux Flux ou seront envoiyer les données
 */
void Ensemble_Cluster::ecrireTab(std::ofstream& flux)
{
    flux << "[";
    for(unsigned int cluster = 0; cluster < this->m_cluster.size(); cluster++)
    {
        flux << "[";
        for(unsigned int proteine = 0; proteine < this->m_cluster.at(cluster)->getnbProtein(); proteine++)
        {
            if((proteine+1) < this->m_cluster.at(cluster)->getnbProtein())
            {
                flux << this->m_cluster.at(cluster)->proteinAt(proteine)->getID()+1 << ",";
            }
            else
            {
                flux << this->m_cluster.at(cluster)->proteinAt(proteine)->getID()+1;
            }
        }
        flux << "]";
        if((cluster+1) < this->m_cluster.size())
        {
            flux << ",";
        }
    }
    flux << "]";
}
